package dev.fitko.fitconnect.core.portal;

import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.domain.model.jwk.ApiJwkSet;
import dev.fitko.fitconnect.api.exceptions.RestApiException;
import dev.fitko.fitconnect.api.services.portal.SelfServicePortalService;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.nio.charset.StandardCharsets;
import java.util.List;

public class SelfServicePortalApiService implements SelfServicePortalService {

    private final RestTemplate restTemplate;
    private final ApplicationConfig config;

    public SelfServicePortalApiService(final ApplicationConfig config, final RestTemplate restTemplate) {
        this.config = config;
        this.restTemplate = restTemplate;
    }

    @Override
    public ApiJwkSet getPublicKeys() {
        final String url = this.config.getWellKnownKeysEndpoint();
        final HttpHeaders headers = getHttpHeaders(MediaType.APPLICATION_JSON);
        final HttpEntity<String> entity = new HttpEntity<>(headers);
        try {
            return this.restTemplate.exchange(url, HttpMethod.GET, entity, ApiJwkSet.class).getBody();
        } catch (final RestClientException e) {
            throw new RestApiException("Well-known keys query failed", e);
        }
    }
    private HttpHeaders getHttpHeaders(final MediaType mediaType) {
        final var headers = new HttpHeaders();
        headers.setContentType(mediaType);
        headers.setAcceptCharset(List.of(StandardCharsets.UTF_8));
        return headers;
    }
}
